export function trusted(fn) {
    return function (...args) {
        var event = args[0];
        if (event.isTrusted) {
            // @ts-ignore
            fn === null || fn === void 0 ? void 0 : fn.apply(this, args);
        }
    };
}
export function self(fn) {
    return function (...args) {
        var event = args[0];
        if (event.target === event.currentTarget) {
            // @ts-ignore
            fn === null || fn === void 0 ? void 0 : fn.apply(this, args);
        }
    };
}
export function stopPropagation(fn) {
    return function (...args) {
        var event = args[0];
        event.stopPropagation();
        // @ts-ignore
        return fn === null || fn === void 0 ? void 0 : fn.apply(this, args);
    };
}
export function once(fn) {
    var ran = false;
    return function (...args) {
        if (ran)
            return;
        ran = true;
        // @ts-ignore
        return fn === null || fn === void 0 ? void 0 : fn.apply(this, args);
    };
}
export function stopImmediatePropagation(fn) {
    return function (...args) {
        var event = args[0];
        event.stopImmediatePropagation();
        // @ts-ignore
        return fn === null || fn === void 0 ? void 0 : fn.apply(this, args);
    };
}
export function preventDefault(fn) {
    return function (...args) {
        var event = args[0];
        event.preventDefault();
        // @ts-ignore
        return fn === null || fn === void 0 ? void 0 : fn.apply(this, args);
    };
}
/**
 * Creates a callable function with event modifiers
 *
 * ```svelte
 * <script>
 *     const onclick = callable(() => {
 *         console.log('clicked!');
 *     }, preventDefault);
 * </script>
 *
 * <a href="/meow" {onclick}>
 *     <span>i won't redirect you!</span>
 * </a>
 * ```
 *
 * @param fn Callback function
 * @param modifiers Event modifiers
 * @returns The callback function
 */
const callable = (fn, ...modifiers) => {
    let acc = fn;
    let arr = modifiers;
    const onceIndex = modifiers.indexOf(once);
    if (onceIndex !== -1) {
        arr.unshift(arr.splice(onceIndex, 1)[0]);
    }
    return arr.reduce((current, modifier) => modifier(current), acc);
};
callable.forEach = function (fn, ...modifiers) {
    const handlers = {};
    return function (key) {
        if (!handlers[key]) {
            handlers[key] = callable(fn, ...modifiers);
        }
        return handlers[key];
    };
};
export default callable;
